@@ -1,19 +1,18 @@
 /*
  * Copyright (C) 2003 Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -133,7 +132,7 @@ parse_legacy_mouse_mode(guint8 *data,
                         gsize len)
 {
         int button = 0;
-        const char *shift = "", *control = "", *meta = "";
+        const char *shift = "", *control = "", *alt = "";
         gboolean motion = FALSE;
         int x, y;
         guint8 b;
@@ -168,8 +167,8 @@ parse_legacy_mouse_mode(guint8 *data,
         shift = b & 4 ?
                 "[shift]" :
                 "";
-        meta = b & 8 ?
-                "[meta]" :
+        alt = b & 8 ?
+                "[alt]" :
                 "";
         control = b & 16 ?
                 "[control]" :
@@ -182,7 +181,7 @@ parse_legacy_mouse_mode(guint8 *data,
                 motion ? "motion " : "",
                 (!motion && button) ? "press" : "",
                 (!motion && !button) ? "release" : "",
-                meta, control, shift,
+                alt, control, shift,
                 x, y);
 
         return 6;
