@@ -395,7 +395,7 @@ sequences = [
                comment='double width double height line: bottom half'),
     seq_ESCAPE('DECSWL', '5', intermediates=(Intermediate.HASH,), flags=Flags.NOP,
                comment='single width single height line'),
-    seq_ESCAPE('DECBI', '6',
+    seq_ESCAPE('DECBI', '6', flags=Flags.NOP,
                comment='back index'),
     seq_ESCAPE('DECDWL', '6', intermediates=(Intermediate.HASH,), flags=Flags.NOP,
                comment='double width single height line'),
@@ -405,7 +405,7 @@ sequences = [
                comment='restore cursor'),
     seq_ESCAPE('DECALN', '8', intermediates=(Intermediate.HASH,),
                comment='screen alignment pattern'),
-    seq_ESCAPE('DECFI', '9',
+    seq_ESCAPE('DECFI', '9', flags=Flags.NOP,
                comment='forward index'),
     seq_ESCAPE('WYDHL_TH', ':', intermediates=(Intermediate.HASH,), flags=Flags.NOP,
                comment='single width double height line: top half'),
@@ -489,11 +489,11 @@ sequences = [
     # CSI sequences
     seq_CSI('ICH', '@',
             comment='insert character'),
-    seq_CSI('SL', '@', intermediates=(Intermediate.SPACE,),
+    seq_CSI('SL', '@', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
             comment='scroll left'),
     seq_CSI('CUU', 'A',
             comment='cursor up'),
-    seq_CSI('SR', 'A', intermediates=(Intermediate.SPACE,),
+    seq_CSI('SR', 'A', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
             comment='scroll right'),
     seq_CSI('CUD', 'B',
             comment='cursor down'),
@@ -581,8 +581,6 @@ sequences = [
             comment='xterm initiate highlight mouse tracking'),
     seq_CSI('DTA', 'T', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
             comment='dimension text area'),
-    seq_CSI('MINTTY_SD', 'T', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
-            comment='mintty unscroll'),
     seq_CSI('XTERM_RTM', 'T', pintro=(ParameterIntro.GT,), flags=Flags.NOP,
             comment='xterm reset title mode'),
     seq_CSI('NP', 'U', flags=Flags.NOP,
@@ -697,7 +695,7 @@ sequences = [
             comment='reset mode dec'),
     seq_CSI('SGR', 'm',
             comment='select graphics rendition'),
-    seq_CSI('DECSGR', 'm', pintro=(ParameterIntro.WHAT,),
+    seq_CSI('DECSGR', 'm', pintro=(ParameterIntro.WHAT,), flags=Flags.NOP,
             comment='DEC select graphics rendition'),
     seq_CSI('XTERM_MODKEYS', 'm', pintro=(ParameterIntro.GT,), flags=Flags.NOP,
             comment='xterm set key modifier options'),
@@ -750,19 +748,19 @@ sequences = [
             comment='select disconnect delay time'),
     seq_CSI('MINTTY_PROGRESS', 'q', intermediates=(Intermediate.PERCENT,), flags=Flags.NOP,
             comment='set progress report'),
+    seq_CSI('DECSR', 'q', intermediates=(Intermediate.MULT,),
+            comment='secure reset'),
     seq_CSI('DECELF', 'q', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
             comment='enable local functions'),
     seq_CSI('DECTID', 'q', intermediates=(Intermediate.COMMA,), flags=Flags.NOP,
             comment='select terminal id'),
     seq_CSI('DECCRTST', 'q', intermediates=(Intermediate.MINUS,), flags=Flags.NOP,
             comment='CRT saver time'),
-    seq_CSI('XTERM_VERSION', 'q', pintro=(ParameterIntro.GT,),
-            comment='request xterm version report'),
     seq_CSI('DECSTBM', 'r',
             comment='set top and bottom margins'),
     seq_CSI('DECSKCV', 'r', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
             comment='set key click volume'),
-    seq_CSI('DECCARA', 'r', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECCARA', 'r', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='change attributes in rectangular area'),
     seq_CSI('DECSCS', 'r', intermediates=(Intermediate.MULT,), flags=Flags.NOP,
             comment='select communication speed'),
@@ -774,7 +772,7 @@ sequences = [
             comment='pcterm'),
     seq_CSI('XTERM_RPM', 'r', pintro=(ParameterIntro.WHAT,),
             comment='xterm restore DEC private mode'),
-    seq_CSI('DECSLRM', 's',
+    seq_CSI('DECSLRM', 's', flags=Flags.NOP,
             comment='set left and right margins'),
     seq_CSI('SCOSC', 's',
             comment='SCO save cursor'),
@@ -794,7 +792,7 @@ sequences = [
             comment='set warning bell volume'),
     seq_CSI('DECSRFR', 't', intermediates=(Intermediate.DQUOTE,), flags=Flags.NOP,
             comment='select refresh rate'),
-    seq_CSI('DECRARA', 't', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECRARA', 't', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='reverse attributes in rectangular area'),
     seq_CSI('XTERM_STM', 't', pintro=(ParameterIntro.GT,), flags=Flags.NOP,
             comment='xterm set title mode'),
@@ -814,19 +812,17 @@ sequences = [
             comment='request user preferred supplemental set'),
     seq_CSI('DECSLCK', 'v', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
             comment='set lock key style'),
-    seq_CSI('DECRQDE', 'v', intermediates=(Intermediate.DQUOTE,),
+    seq_CSI('DECRQDE', 'v', intermediates=(Intermediate.DQUOTE,), flags=Flags.NOP,
             comment='request display extent'),
-    seq_CSI('DECCRA', 'v', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECCRA', 'v', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='copy rectangular area'),
     seq_CSI('DECRPKT', 'v', intermediates=(Intermediate.COMMA,), flags=Flags.NOP,
-            direction = Direction.TTH,
             comment='report key type'),
     seq_CSI('WYCAA', 'w', flags=Flags.NOP,
             comment='redefine character display attribute association'),
-    seq_CSI('DECRPDE', 'w', intermediates=(Intermediate.DQUOTE,),
-            direction = Direction.TTH,
+    seq_CSI('DECRPDE', 'w', intermediates=(Intermediate.DQUOTE,), flags=Flags.NOP,
             comment='report displayed extent'),
-    seq_CSI('DECRQPSR', 'w', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECRQPSR', 'w', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='request presentation state report'),
     seq_CSI('DECEFR', 'w', intermediates=(Intermediate.SQUOTE,), flags=Flags.NOP,
             comment='enable filter rectangle'),
@@ -836,11 +832,11 @@ sequences = [
             comment='request terminal parameters'),
     seq_CSI('WYCDIR', 'x', flags=Flags.NOP,
             comment='set current character attributes'),
-    seq_CSI('DECFRA', 'x', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECFRA', 'x', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='fill rectangular area'),
     seq_CSI('DECES', 'x', intermediates=(Intermediate.AND,), flags=Flags.NOP,
             comment='enable session'),
-    seq_CSI('DECSACE', 'x', intermediates=(Intermediate.MULT,),
+    seq_CSI('DECSACE', 'x', intermediates=(Intermediate.MULT,), flags=Flags.NOP,
             comment='select attribute change extent'),
     seq_CSI('DECRQPKFM', 'x', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
             comment='request program key free memory'),
@@ -858,7 +854,7 @@ sequences = [
             comment='update session'),
     seq_CSI('WYSCRATE', 'z', flags=Flags.NOP,
             comment='set smooth scroll rate'),
-    seq_CSI('DECERA', 'z', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECERA', 'z', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='erase rectangular area'),
     seq_CSI('DECELR', 'z', intermediates=(Intermediate.SQUOTE,), flags=Flags.NOP,
             comment='enable locator reporting'),
@@ -870,7 +866,7 @@ sequences = [
             comment='down line load allocation'),
     seq_CSI('XTERM_PUSHSGR', '{', intermediates=(Intermediate.HASH,), flags=Flags.NOP,
             comment='xterm push SGR stack'),
-    seq_CSI('DECSERA', '{', intermediates=(Intermediate.CASH,),
+    seq_CSI('DECSERA', '{', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='selective erase rectangular area'),
     seq_CSI('DECSLE', '{', intermediates=(Intermediate.SQUOTE,), flags=Flags.NOP,
             comment='select locator events'),
@@ -878,7 +874,7 @@ sequences = [
             comment='select color lookup table'),
     seq_CSI('DECSZS', '{', intermediates=(Intermediate.COMMA,), flags=Flags.NOP,
             comment='select zero symbol'),
-    seq_CSI('XTERM_REPORTSGR', '|', intermediates=(Intermediate.HASH,),
+    seq_CSI('XTERM_REPORTSGR', '|', intermediates=(Intermediate.HASH,), flags=Flags.NOP,
             comment='xterm SGR report'),
     seq_CSI('DECSCPP', '|', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='select columns per page'),
@@ -894,17 +890,17 @@ sequences = [
             comment='xterm pop SGR stack'),
     seq_CSI('DECSASD', '}', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='select active status display'),
-    seq_CSI('DECIC', '}', intermediates=(Intermediate.SQUOTE,),
+    seq_CSI('DECIC', '}', intermediates=(Intermediate.SQUOTE,), flags=Flags.NOP,
             comment='insert column'),
     seq_CSI('DECATC', '}', intermediates=(Intermediate.COMMA,), flags=Flags.NOP,
             comment='alternate text color'),
-    seq_CSI('DECFNK', '~', direction=Direction.BIDI, flags=Flags.NOP,
+    seq_CSI('DECFNK', '~', direction=Direction.TTH, flags=Flags.NOP,
             comment='dec function key / XTERM bracketed paste'),
     seq_CSI('DECTME', '~', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
             comment='terminal mode emulation'),
     seq_CSI('DECSSDT', '~', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='select status display line type'),
-    seq_CSI('DECDC', '~', intermediates=(Intermediate.SQUOTE,),
+    seq_CSI('DECDC', '~', intermediates=(Intermediate.SQUOTE,), flags=Flags.NOP,
             comment='delete column'),
     seq_CSI('DECPS', '~', intermediates=(Intermediate.COMMA,), flags=Flags.NOP,
             comment='play sound'),
@@ -924,25 +920,18 @@ sequences = [
             comment='SIXEL graphics'),
     seq_DCS('DECRQSS', 'q', intermediates=(Intermediate.CASH,),
             comment='request selection or setting'),
-    seq_DCS('XTERM_RQTCAP', 'q', intermediates=(Intermediate.PLUS,),
+    seq_DCS('XTERM_RQTCAP', 'q', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
             comment='xterm request termcap/terminfo'),
     seq_DCS('DECLBAN', 'r', flags=Flags.NOP,
             comment='load banner message'),
-    seq_DCS('XTERM_TCAPR', 'r', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
-            direction=Direction.TTH,
-            comment='xterm termcap report'),
-    seq_DCS('DECTSR', 's', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
-            direction = Direction.TTH,
-            comment='terminal state report'),
+    seq_DCS('DECRQTSR', 's', intermediates=(Intermediate.CASH,),
+            comment='request terminal state report'),
     seq_DCS('XDGSYNC', 's', pintro=(ParameterIntro.EQUAL,), flags=Flags.NOP,
             comment='synchronous update'),
     seq_DCS('DECRSPS', 't', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='restore presentation state'),
     seq_DCS('DECAUPSS', 'u', intermediates=(Intermediate.BANG,), flags=Flags.NOP,
             comment='assign user preferred supplemental sets'),
-    seq_DCS('DECPSR', 'u', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
-            direction = Direction.TTH,
-            comment='presentation state report'),
     seq_DCS('DECLANS', 'v', flags=Flags.NOP,
             comment='load answerback message'),
     seq_DCS('DECLBD', 'w', flags=Flags.NOP,
@@ -964,10 +953,8 @@ sequences = [
     seq_DCS('WYLSFNT', '}', flags=Flags.NOP,
             comment='load soft font'),
     seq_DCS('DECRPFK', '}', intermediates=(Intermediate.DQUOTE,), flags=Flags.NOP,
-            direction=Direction.TTH,
             comment='report function key definition'),
     seq_DCS('DECRPAK', '~', intermediates=(Intermediate.DQUOTE,), flags=Flags.NOP,
-            direction=Direction.TTH,
             comment='report all modifier/alphanumeric key state'),
 
     # SCI sequences
@@ -997,7 +984,7 @@ def get_sequences(predicate):
     return result
 
 '''
-Returns: Dict[str, Tuple[Flags, Direction, str]] mapping command name to (flags, direction, comment)
+Returns: Dict[str, Tuple[str, str]] mapping command name to (comment, flags)
 '''
 def get_commands(predicate):
     cmds={}
@@ -1007,16 +994,14 @@ def get_commands(predicate):
         seq_list = all_seqs[_seq]
         for seq in seq_list:
             if seq.name in cmds:
-                flags, direction, comment = cmds[seq.name]
+                flags, comment = cmds[seq.name]
 
                 if flags != seq.flags:
                     raise ValueError(f'{seq.name} flags inconsistent: {seq.flags} vs {flags}')
-                if direction != seq.direction:
-                    raise ValueError(f'{seq.name} direction inconsistent: {seq.direction} vs {direction}')
                 if comment != seq.comment:
                     raise ValueError(f'{seq.name} comment inconsistent: {seq.comment} vs {comment}')
 
-            cmds[seq.name]=(seq.flags, seq.direction, seq.comment)
+            cmds[seq.name]=(seq.flags, seq.comment)
 
         # Add an extra entry for the disambiguation command
         if len(seq_list) > 1:
@@ -1026,7 +1011,6 @@ def get_commands(predicate):
             or_comment=' or '.join(tuple([seq.comment for seq in sorted_seqs]))
 
             or_flags=None
-            or_direction=Direction.HTT
             for seq in sorted_seqs:
                 flags=seq.flags
                 if flags is None or or_flags is None:
@@ -1034,9 +1018,7 @@ def get_commands(predicate):
                 else:
                     or_flags &= flags
 
-                or_direction |= seq.direction
-
-            cmds[or_name] = (or_flags, or_direction, or_comment)
+            cmds[or_name] = (or_flags, or_comment)
 
     return cmds
 
@@ -1068,9 +1050,8 @@ def get_seqs(predicate):
             name=seq_list[0].name
             comment=seq_list[0].comment
             flags=seq_list[0].flags
-            direction=seq_list[0].direction
 
-        seqs[seq] = (seq.stype, name, seq.final, seq.pintro, seq.intermediates, flags, direction, comment)
+        seqs[seq] = (seq.stype, name, seq.final, seq.pintro, seq.intermediates, flags, comment)
 
     return seqs
 
@@ -1100,7 +1081,7 @@ def write_header(outfile):
 
 
 ''' Write sequences '''
-def write_seqs(output, stype, sdir=Direction.HTT):
+def write_seqs(output, stype):
     outfile = open(output.as_posix(), 'w')
     write_header(outfile)
     outfile.write('''
@@ -1131,7 +1112,7 @@ def write_seqs(output, stype, sdir=Direction.HTT):
 
     seqs = get_seqs(lambda seq: seq.stype == stype)
     for seq in seqs:
-        stype, name, final, pintro, intermediates, flags, direction, comment = seqs[seq]
+        stype, name, final, pintro, intermediates, flags, comment = seqs[seq]
 
         if len(intermediates) > 2:
             raise ValueError('{name} has too many intermediates')
@@ -1150,9 +1131,6 @@ def write_seqs(output, stype, sdir=Direction.HTT):
         else:
             macro = '_VTE_SEQ'
 
-        if not direction & sdir:
-            continue
-
         outfile.write(f'{macro}('
                       f'{name}, '
                       f'{stype.name}, '
@@ -1183,10 +1161,7 @@ _VTE_CMD(GRAPHIC) /* graphics character */
 
     cmds = get_commands(lambda seq: True)
     for name in sorted(cmds):
-        flags, direction, comment = cmds[name]
-        if not direction & Direction.HTT:
-            continue
-
+        flags, comment = cmds[name]
         if flags is None or not (flags & Flags.NOP):
             if comment is not None:
                 outfile.write(f'_VTE_CMD({name}) /* {comment} */\n')
@@ -1195,10 +1170,7 @@ _VTE_CMD(GRAPHIC) /* graphics character */
 
     outfile.write('/* Unimplemented in VTE: */\n')
     for name in sorted(cmds):
-        flags, direction, comment = cmds[name]
-        if not direction & Direction.HTT:
-            continue
-
+        flags, comment = cmds[name]
         if flags is not None and flags & Flags.NOP:
             if comment is not None:
                 outfile.write(f'_VTE_NOP({name}) /* {comment} */\n')
@@ -1234,9 +1206,7 @@ _VTE_CMD_HANDLER(GRAPHIC) /* graphics character */
 
     cmds = get_commands(lambda seq: True)
     for name in sorted(cmds):
-        flags, direction, comment = cmds[name]
-        if not direction & Direction.HTT:
-            continue
+        flags, comment = cmds[name]
         if comment is not None:
             outfile.write(f'{cmd_handler_macro(flags)}({name}) /* {comment} */\n')
         else:
