@@ -21,12 +21,13 @@
 #include <gtk/gtk.h>
 #include <gtk/gtk-a11y.h>
 #include <string.h>
+#include "debug.h"
 #include <vte/vte.h>
 #include "vteaccess.h"
 #include "vteinternal.hh"
 #include "widget.hh"
 
-#if __has_include(<locale.h>)
+#ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 #include <glib/gi18n-lib.h>
@@ -47,7 +48,7 @@ typedef struct _VteTerminalAccessiblePrivate {
 	gboolean snapshot_caret_invalid;	/* This data is stale. */
 	GString *snapshot_text;		/* Pointer to UTF-8 text. */
 	GArray *snapshot_characters;	/* Offsets to character begin points. */
-        VteCharAttrList snapshot_attributes; /* Attributes, per byte. */
+	GArray *snapshot_attributes;	/* Attributes, per byte. */
 	GArray *snapshot_linebreaks;	/* Offsets to line breaks. */
 	gint snapshot_caret;       /* Location of the cursor (in characters). */
         gboolean text_caret_moved_pending;
@@ -141,6 +142,9 @@ xy_from_offset (VteTerminalAccessiblePrivate *priv,
 static void
 emit_text_caret_moved(GObject *object, glong caret)
 {
+	_vte_debug_print(VTE_DEBUG_SIGNALS|VTE_DEBUG_ALLY,
+			"Accessibility peer emitting "
+			"`text-caret-moved'.\n");
 	g_signal_emit_by_name(object, "text-caret-moved", caret);
 }
 
@@ -155,6 +159,12 @@ emit_text_changed_insert(GObject *object,
 	/* Convert the byte offsets to character offsets. */
 	start = g_utf8_pointer_to_offset (text, text + offset);
 	count = g_utf8_pointer_to_offset (text + offset, text + offset + len);
+	_vte_debug_print(VTE_DEBUG_SIGNALS|VTE_DEBUG_ALLY,
+			"Accessibility peer emitting "
+			"`text-changed::insert' (%ld, %ld) (%ld, %ld).\n"
+			"Inserted text was `%.*s'.\n",
+			offset, len, start, count,
+			(int) len, text + offset);
 	g_signal_emit_by_name(object, "text-changed::insert", start, count);
 }
 
@@ -169,6 +179,12 @@ emit_text_changed_delete(GObject *object,
 	/* Convert the byte offsets to characters. */
 	start = g_utf8_pointer_to_offset (text, text + offset);
 	count = g_utf8_pointer_to_offset (text + offset, text + offset + len);
+	_vte_debug_print(VTE_DEBUG_SIGNALS|VTE_DEBUG_ALLY,
+			"Accessibility peer emitting "
+			"`text-changed::delete' (%ld, %ld) (%ld, %ld).\n"
+			"Deleted text was `%.*s'.\n",
+			offset, len, start, count,
+			(int) len, text + offset);
 	g_signal_emit_by_name(object, "text-changed::delete", start, count);
 }
 
@@ -245,7 +261,11 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 
 		/* Free the attribute lists and allocate a new array to hold
 		 * them. */
-                vte_char_attr_list_set_size(&priv->snapshot_attributes, 0);
+		if (priv->snapshot_attributes != NULL) {
+			g_array_free(priv->snapshot_attributes, TRUE);
+		}
+		priv->snapshot_attributes = g_array_new(FALSE, FALSE,
+							sizeof(struct _VteCharAttributes));
 
 		/* Free the linebreak offsets and allocate a new array to hold
 		 * them. */
@@ -255,18 +275,16 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 		priv->snapshot_linebreaks = g_array_new(FALSE, FALSE, sizeof(int));
 
 		/* Get a new view of the uber-label. */
-                auto text = g_string_new(nullptr);
                 try {
-                        impl->get_text_displayed_a11y(text,
-                                                      &priv->snapshot_attributes);
+                        priv->snapshot_text = impl->get_text_displayed_a11y(true /* wrap */,
+                                                                            priv->snapshot_attributes);
                 } catch (...) {
-                        g_string_truncate(text, 0);
+                        priv->snapshot_text = g_string_new("");
                 }
-                priv->snapshot_text = text;
 		/* Get the offsets to the beginnings of each character. */
 		i = 0;
 		next = priv->snapshot_text->str;
-		while (i < vte_char_attr_list_get_size(&priv->snapshot_attributes)) {
+		while (i < priv->snapshot_attributes->len) {
 			g_array_append_val(priv->snapshot_characters, i);
 			next = g_utf8_next_char(next);
 			if (next == NULL) {
@@ -280,12 +298,18 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 			/* Get the attributes for the current cell. */
 			offset = g_array_index(priv->snapshot_characters,
 					       int, i);
-                        attrs = *vte_char_attr_list_get(&priv->snapshot_attributes, offset);
+			attrs = g_array_index(priv->snapshot_attributes,
+					      struct _VteCharAttributes,
+					      offset);
 			/* If this character is on a row different from the row
 			 * the character we looked at previously was on, then
 			 * it's a new line and we need to keep track of where
 			 * it is. */
 			if ((i == 0) || (attrs.row != row)) {
+				_vte_debug_print(VTE_DEBUG_ALLY,
+						"Row %d/%ld begins at %u.\n",
+						priv->snapshot_linebreaks->len,
+						attrs.row, i);
 				g_array_append_val(priv->snapshot_linebreaks, i);
 			}
 			row = attrs.row;
@@ -298,6 +322,8 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 
 	/* Update the caret position. */
 	vte_terminal_get_cursor_position(terminal, &ccol, &crow);
+	_vte_debug_print(VTE_DEBUG_ALLY,
+			"Cursor at (%ld, " "%ld).\n", ccol, crow);
 
 	/* Get the offsets to the beginnings of each line. */
 	caret = 0;
@@ -305,7 +331,9 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 		/* Get the attributes for the current cell. */
 		offset = g_array_index(priv->snapshot_characters,
 				       int, i);
-                attrs = *vte_char_attr_list_get(&priv->snapshot_attributes, offset);
+		attrs = g_array_index(priv->snapshot_attributes,
+				      struct _VteCharAttributes,
+				      offset);
 		/* If this cell is "before" the cursor, move the
 		 * caret to be "here". */
 		if ((attrs.row < crow) ||
@@ -323,6 +351,12 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 
 	/* Done updating the caret position, whether we needed to or not. */
 	priv->snapshot_caret_invalid = FALSE;
+
+	_vte_debug_print(VTE_DEBUG_ALLY,
+			"Refreshed accessibility snapshot, "
+			"%ld cells, %ld characters.\n",
+			(long)priv->snapshot_attributes->len,
+			(long)priv->snapshot_characters->len);
 }
 
 static void
@@ -358,12 +392,6 @@ _vte_terminal_accessible_text_modified(VteTerminalAccessible* accessible)
 	glong offset, caret_offset, olen, clen;
 	gint old_snapshot_caret;
 
-        auto widget = gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible));
-        auto terminal = VTE_TERMINAL(widget);
-
-        if (!vte_terminal_get_enable_a11y (terminal))
-                return;
-
 	old_snapshot_caret = priv->snapshot_caret;
 	priv->snapshot_contents_invalid = TRUE;
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
@@ -465,44 +493,6 @@ _vte_terminal_accessible_text_modified(VteTerminalAccessible* accessible)
         g_array_free(old_characters, TRUE);
 }
 
-/* Compute a simple diff between the `before` string (before_len bytes) and the
-   `after` string (after_len bytes). We here only check for common head and
-   tail.
-
-   If the strings are equal, this returns FALSE.
-
-   If the strings differ, this returns TRUE, and *head_len is set to the number
-   of bytes that are identical at the beginning of `before` and `after`, and
-   *tail_len is set to the number of bytes that are identical at the end of
-   `before` and `after`. */
-static gboolean
-check_diff(const gchar *before, guint before_len,
-	   const gchar *after, guint after_len,
-	   guint *head_len, guint *tail_len)
-{
-	guint i;
-
-	for (i = 0; i < before_len && i < after_len; i++)
-		if (before[i] != after[i])
-			break;
-
-	if (i == before_len && i == after_len)
-		/* They are identical. */
-		return FALSE;
-
-	/* They started diverging at i. */
-	*head_len = i;
-
-	for (i = 1; i <= before_len - *head_len && i <= after_len - *head_len; i++)
-		if (before[before_len - i] != after[after_len - i])
-			break;
-
-	/* They finished diverging here. */
-	*tail_len = i - 1;
-
-	return TRUE;
-}
-
 void
 _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
                                        long howmuch)
@@ -510,7 +500,7 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 	struct _VteCharAttributes attr;
 	long delta, row_count;
-	guint drop, old_len, new_len, old_common, new_common;
+	guint i, len;
 
         /* TODOegmont: Fix this for smooth scrolling */
         /* g_assert(howmuch != 0); */
@@ -519,9 +509,6 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
         auto widget = gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible));
         auto terminal = VTE_TERMINAL(widget);
 
-        if (!vte_terminal_get_enable_a11y (terminal))
-                return;
-
         row_count = vte_terminal_get_row_count(terminal);
 	if (((howmuch < 0) && (howmuch <= -row_count)) ||
 	    ((howmuch > 0) && (howmuch >= row_count))) {
@@ -550,217 +537,104 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
                 vte_terminal_accessible_maybe_emit_text_caret_moved(accessible);
 		return;
 	}
-
 	/* Find the start point. */
 	delta = 0;
-        if (vte_char_attr_list_get_size(&priv->snapshot_attributes) > 0) {
-                attr = *vte_char_attr_list_get(&priv->snapshot_attributes, 0);
-                delta = attr.row;
-        }
+	if (priv->snapshot_attributes != NULL) {
+		if (priv->snapshot_attributes->len > 0) {
+			attr = g_array_index(priv->snapshot_attributes,
+					     struct _VteCharAttributes,
+					     0);
+			delta = attr.row;
+		}
+	}
 	/* We scrolled up, so text was added at the top and removed
 	 * from the bottom. */
 	if ((howmuch < 0) && (howmuch > -row_count)) {
+		gboolean inserted = FALSE;
 		howmuch = -howmuch;
-		if (priv->snapshot_text != NULL) {
-			old_len = vte_char_attr_list_get_size(&priv->snapshot_attributes);
-
+		if (priv->snapshot_attributes != NULL &&
+				priv->snapshot_text != NULL) {
 			/* Find the first byte that scrolled off. */
-			for (old_common = 0; old_common < old_len; old_common++) {
-				attr = *vte_char_attr_list_get(&priv->snapshot_attributes, old_common);
+			for (i = 0; i < priv->snapshot_attributes->len; i++) {
+				attr = g_array_index(priv->snapshot_attributes,
+						struct _VteCharAttributes,
+						i);
 				if (attr.row >= delta + row_count - howmuch) {
 					break;
 				}
 			}
-			if (old_common < old_len) {
-				drop = old_len - old_common;
-
-				GString *old_text;
-				GArray *old_characters;
-
-				/* Refresh. */
-				priv->snapshot_contents_invalid = TRUE;
-				vte_terminal_accessible_update_private_data_if_needed(accessible,
-										      &old_text,
-										      &old_characters);
-
-				GString *new_text = priv->snapshot_text;
-				GArray *new_characters = priv->snapshot_characters;
-				new_len = new_text->len;
-
-				if (old_common > new_len)
-					/* Not only did we scroll, but we even removed some text */
-					new_common = new_len;
-				else
-					new_common = old_common;
-
-				guint head_len, tail_len;
-				gboolean diff;
-
-				/* Check for more difference than just the addition at top and removal at bottom */
-				diff = check_diff(old_text->str, old_common,
-						  new_text->str + new_len - new_common, new_common,
-						  &head_len, &tail_len);
-
-				/* Note: to keep offsets correct, we need to
-				 * notify deletions from bottom to top and
-				 * additions from top to bottom
-				 */
-
-				/* Temporarily switch to old text to emit deletion events */
-				priv->snapshot_text = old_text;
-				priv->snapshot_characters = old_characters;
-
-				/* Delete bottom first */
+			if (i < priv->snapshot_attributes->len) {
+				/* The rest of the string was deleted -- make a note. */
 				emit_text_changed_delete(G_OBJECT(accessible),
-						old_text->str,
-						old_common,
-						drop);
-
-				if (diff)
-					/* Not only did we scroll, but we also modified some bits */
-					/* Delete middle */
-					emit_text_changed_delete(G_OBJECT(accessible),
-								 old_text->str,
-								 head_len,
-								 old_common - tail_len - head_len);
-
-				/* Switch to new text */
-				priv->snapshot_text = new_text;
-				priv->snapshot_characters = new_characters;
-
-				g_string_free(old_text, TRUE);
-				g_array_free(old_characters, TRUE);
-
-				/* If we now have more text than before, the initial portion
-				 * was added. */
-				if (new_len > new_common)
-					/* Add top */
-					emit_text_changed_insert(G_OBJECT(accessible),
-								 new_text->str,
-								 0,
-								 new_len - new_common);
-
-				if (diff)
-					/* Add middle */
-					emit_text_changed_insert(G_OBJECT(accessible),
-								 new_text->str,
-								 new_len - new_common + head_len,
-								 new_common - tail_len - head_len);
+						priv->snapshot_text->str,
+						i,
+						priv->snapshot_attributes->len - i);
+			}
+			inserted = TRUE;
+		}
+		/* Refresh.  Note that i is now the length of the data which
+		 * we expect to have left over. */
+		priv->snapshot_contents_invalid = TRUE;
+		vte_terminal_accessible_update_private_data_if_needed(accessible,
+								      NULL,
+								      NULL);
+		/* If we now have more text than before, the initial portion
+		 * was added. */
+		if (inserted) {
+			len = priv->snapshot_text->len;
+			if (len > i) {
+				emit_text_changed_insert(G_OBJECT(accessible),
+							 priv->snapshot_text->str,
+							 0,
+							 len - i);
 			}
-		} else {
-			/* Just refresh. */
-			priv->snapshot_contents_invalid = TRUE;
-			vte_terminal_accessible_update_private_data_if_needed(accessible,
-									      NULL,
-									      NULL);
 		}
-
                 vte_terminal_accessible_maybe_emit_text_caret_moved(accessible);
 		return;
 	}
 	/* We scrolled down, so text was added at the bottom and removed
 	 * from the top. */
 	if ((howmuch > 0) && (howmuch < row_count)) {
-		if (priv->snapshot_text != NULL) {
-			/* Leave trailing '\n' untouched, so that the exposed text always contains a trailing '\n',
-			   insertion happens in front of it: bug 657960 */
-			old_len = vte_char_attr_list_get_size(&priv->snapshot_attributes) - 1;
-
+		gboolean inserted = FALSE;
+		if (priv->snapshot_attributes != NULL &&
+				priv->snapshot_text != NULL) {
 			/* Find the first byte that wasn't scrolled off the top. */
-			for (drop = 0; drop < old_len; drop++) {
-				attr = *vte_char_attr_list_get(&priv->snapshot_attributes, drop);
+			for (i = 0; i < priv->snapshot_attributes->len; i++) {
+				attr = g_array_index(priv->snapshot_attributes,
+						struct _VteCharAttributes,
+						i);
 				if (attr.row >= delta + howmuch) {
 					break;
 				}
 			}
-
-			/* Figure out how much text was common, and refresh. */
-			old_common = old_len - drop;
-
-			GString *old_text;
-			GArray *old_characters;
-
-			priv->snapshot_contents_invalid = TRUE;
-			vte_terminal_accessible_update_private_data_if_needed(accessible,
-									      &old_text,
-									      &old_characters);
-
-			GString *new_text = priv->snapshot_text;
-			GArray *new_characters = priv->snapshot_characters;
-			new_len = new_text->len - 1;
-
-			if (old_common > new_len)
-				/* Not only did we scroll, but we even removed some text */
-				new_common = new_len;
-			else
-				new_common = old_common;
-
-			guint head_len, tail_len;
-			guint extra_add = 0;
-			gboolean diff;
-
-			/* Check for more difference than just the removal at top and addition at bottom */
-			diff = check_diff(old_text->str + drop, old_common,
-				       new_text->str, new_common,
-				       &head_len, &tail_len);
-
-			/* Note: to keep offsets correct, we need to
-			 * notify deletions from bottom to top and
-			 * additions from top to bottom
-			 */
-
-			/* Temporarily switch to old text to emit deletion events */
-			priv->snapshot_text = old_text;
-			priv->snapshot_characters = old_characters;
-
-			if (diff)
-				/* Not only did we scroll, but we also modified some bits */
-				/* Delete middle first */
-				emit_text_changed_delete(G_OBJECT(accessible),
-							 old_text->str,
-							 drop + head_len,
-							 old_common - tail_len - head_len);
-
-			/* Delete top */
+			/* That many bytes disappeared -- make a note. */
 			emit_text_changed_delete(G_OBJECT(accessible),
-					old_text->str,
+					priv->snapshot_text->str,
 					0,
-                                        drop);
-
-			/* Switch to new text */
-			priv->snapshot_text = new_text;
-			priv->snapshot_characters = new_characters;
-
-			g_string_free(old_text, TRUE);
-			g_array_free(old_characters, TRUE);
-
-			if (diff)
-			{
-				/* Add middle */
-				if (tail_len)
-					emit_text_changed_insert(G_OBJECT(accessible),
-								 new_text->str,
-								 head_len,
-								 new_common - head_len - tail_len);
-				else
-					/* Diff was at bottom, merge with addition below */
-					extra_add = new_common - head_len - tail_len;
-			}
-
-			/* Add bottom */
-			if (new_len > new_common - extra_add)
+					i);
+			/* Figure out how much text was left, and refresh. */
+			i = strlen(priv->snapshot_text->str + i);
+			inserted = TRUE;
+		}
+		priv->snapshot_contents_invalid = TRUE;
+		vte_terminal_accessible_update_private_data_if_needed(accessible,
+								      NULL,
+								      NULL);
+		/* Any newly-added string data is new, so note that it was
+		 * inserted. */
+		if (inserted) {
+			len = priv->snapshot_text->len;
+			if (len > i) {
+				/* snapshot_text always contains a trailing '\n',
+				 * insertion happens in front of it: bug 657960 */
+				// g_assert(i >= 1);
+                            if (i > 0)
 				emit_text_changed_insert(G_OBJECT(accessible),
 							 priv->snapshot_text->str,
-							 new_common - extra_add,
-							 new_len - (new_common - extra_add));
-		} else {
-			/* Just refresh. */
-			priv->snapshot_contents_invalid = TRUE;
-			vte_terminal_accessible_update_private_data_if_needed(accessible,
-									      NULL,
-									      NULL);
+							 i - 1,
+							 len - i);
+			}
 		}
-
                 vte_terminal_accessible_maybe_emit_text_caret_moved(accessible);
 		return;
 	}
@@ -774,9 +648,8 @@ vte_terminal_accessible_invalidate_cursor(VteTerminal *terminal, gpointer data)
         VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 
-        if (!vte_terminal_get_enable_a11y (terminal))
-                return;
-
+	_vte_debug_print(VTE_DEBUG_ALLY,
+			"Invalidating accessibility cursor.\n");
 	priv->snapshot_caret_invalid = TRUE;
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
@@ -789,11 +662,7 @@ vte_terminal_accessible_title_changed(VteTerminal *terminal, gpointer data)
 {
         VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 
-        if (!vte_terminal_get_enable_a11y (terminal))
-                return;
-
-        auto const title = vte_terminal_get_window_title(terminal);
-        atk_object_set_description(ATK_OBJECT(accessible), title ? title : "");
+	atk_object_set_description(ATK_OBJECT(accessible), vte_terminal_get_window_title(terminal));
 }
 
 /* Reflect visibility-notify events. */
@@ -806,9 +675,6 @@ vte_terminal_accessible_visibility_notify(VteTerminal *terminal,
 	GtkWidget *widget;
 	gboolean visible;
 
-        if (!vte_terminal_get_enable_a11y (terminal))
-                return FALSE;
-
 	visible = event->state != GDK_VISIBILITY_FULLY_OBSCURED;
 	/* The VISIBLE state indicates that this widget is "visible". */
 	atk_object_notify_state_change(ATK_OBJECT(accessible),
@@ -840,9 +706,6 @@ vte_terminal_accessible_selection_changed (VteTerminal *terminal,
 {
         VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 
-        if (!vte_terminal_get_enable_a11y (terminal))
-                return;
-
 	g_signal_emit_by_name (accessible, "text_selection_changed");
 }
 
@@ -892,15 +755,16 @@ _vte_terminal_accessible_init (VteTerminalAccessible *accessible)
 {
         VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private (accessible);
 
+	_vte_debug_print(VTE_DEBUG_ALLY, "Initialising accessible peer.\n");
+
 	priv->snapshot_text = NULL;
 	priv->snapshot_characters = NULL;
+	priv->snapshot_attributes = NULL;
 	priv->snapshot_linebreaks = NULL;
 	priv->snapshot_caret = -1;
 	priv->snapshot_contents_invalid = TRUE;
 	priv->snapshot_caret_invalid = TRUE;
         priv->text_caret_moved_pending = FALSE;
-
-	vte_char_attr_list_init(&priv->snapshot_attributes);
 }
 
 static void
@@ -911,6 +775,8 @@ vte_terminal_accessible_finalize(GObject *object)
         GtkWidget *widget;
 	gint i;
 
+	_vte_debug_print(VTE_DEBUG_ALLY, "Finalizing accessible peer.\n");
+
         widget = gtk_accessible_get_widget (GTK_ACCESSIBLE(accessible));
 
 	if (widget != NULL) {
@@ -944,7 +810,9 @@ vte_terminal_accessible_finalize(GObject *object)
 	if (priv->snapshot_characters != NULL) {
 		g_array_free(priv->snapshot_characters, TRUE);
 	}
-        vte_char_attr_list_clear(&priv->snapshot_attributes);
+	if (priv->snapshot_attributes != NULL) {
+		g_array_free(priv->snapshot_attributes, TRUE);
+	}
 	if (priv->snapshot_linebreaks != NULL) {
 		g_array_free(priv->snapshot_linebreaks, TRUE);
 	}
@@ -980,6 +848,11 @@ vte_terminal_accessible_get_text(AtkText *text,
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
 
+	_vte_debug_print(VTE_DEBUG_ALLY,
+			"Getting text from %d to %d of %d.\n",
+			start_offset, end_offset,
+			priv->snapshot_characters->len);
+
 	/* If the requested area is after all of the text, just return an
 	 * empty string. */
 	if (start_offset >= (int) priv->snapshot_characters->len) {
@@ -1027,6 +900,18 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 
         auto impl = IMPL_FROM_ACCESSIBLE(text);
 
+	_vte_debug_print(VTE_DEBUG_ALLY,
+			"Getting %s %s at %d of %d.\n",
+			(direction == direction_current) ? "this" :
+			((direction == direction_next) ? "next" : "previous"),
+			(boundary_type == ATK_TEXT_BOUNDARY_CHAR) ? "char" :
+			((boundary_type == ATK_TEXT_BOUNDARY_LINE_START) ? "line (start)" :
+			((boundary_type == ATK_TEXT_BOUNDARY_LINE_END) ? "line (end)" :
+			((boundary_type == ATK_TEXT_BOUNDARY_WORD_START) ? "word (start)" :
+			((boundary_type == ATK_TEXT_BOUNDARY_WORD_END) ? "word (end)" :
+			((boundary_type == ATK_TEXT_BOUNDARY_SENTENCE_START) ? "sentence (start)" :
+			((boundary_type == ATK_TEXT_BOUNDARY_SENTENCE_END) ? "sentence (end)" : "unknown")))))),
+			offset, priv->snapshot_attributes->len);
 	g_assert(priv->snapshot_text != NULL);
 	g_assert(priv->snapshot_characters != NULL);
 	if (offset >= (int) priv->snapshot_characters->len) {
@@ -1041,7 +926,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			 * position, the one before it, or the one after it. */
 			offset += direction;
 			start = MAX(offset, 0);
-			end = MIN(offset + 1, (int) vte_char_attr_list_get_size(&priv->snapshot_attributes));
+			end = MIN(offset + 1, (int) priv->snapshot_attributes->len);
 			break;
 		case ATK_TEXT_BOUNDARY_WORD_START:
 			/* Back up to the previous non-word-word transition. */
@@ -1218,6 +1103,9 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 					break;
 				}
 			}
+			_vte_debug_print(VTE_DEBUG_ALLY,
+					"Character %d is on line %d.\n",
+					offset, line);
 			/* Perturb the line number to handle before/at/after. */
 			line += direction;
 			line = MIN(line, priv->snapshot_linebreaks->len - 1);
@@ -1228,6 +1116,9 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			line = MIN(line, priv->snapshot_linebreaks->len - 1);
 			end = g_array_index(priv->snapshot_linebreaks,
 						    int, line);
+			_vte_debug_print(VTE_DEBUG_ALLY,
+					"Line runs from %d to %d.\n",
+					start, end);
 			break;
 		case ATK_TEXT_BOUNDARY_SENTENCE_START:
 		case ATK_TEXT_BOUNDARY_SENTENCE_END:
@@ -1299,9 +1190,7 @@ static gunichar
 vte_terminal_accessible_get_character_at_offset(AtkText *text, gint offset)
 {
         VteTerminalAccessible *accessible = VTE_TERMINAL_ACCESSIBLE(text);
-#ifndef G_DISABLE_ASSERT
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
-#endif
 	char *unichar;
 	gunichar ret;
 
@@ -1384,10 +1273,14 @@ vte_terminal_accessible_get_run_attributes(AtkText *text, gint offset,
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
 
-        attr = *vte_char_attr_list_get(&priv->snapshot_attributes, offset);
+	attr = g_array_index (priv->snapshot_attributes,
+			      struct _VteCharAttributes,
+			      offset);
 	*start_offset = 0;
 	for (i = offset; i--;) {
-                cur_attr = *vte_char_attr_list_get(&priv->snapshot_attributes, i);
+		cur_attr = g_array_index (priv->snapshot_attributes,
+				      struct _VteCharAttributes,
+				      i);
 		if (!_pango_color_equal (&cur_attr.fore, &attr.fore) ||
 		    !_pango_color_equal (&cur_attr.back, &attr.back) ||
 		    cur_attr.underline != attr.underline ||
@@ -1396,9 +1289,11 @@ vte_terminal_accessible_get_run_attributes(AtkText *text, gint offset,
 			break;
 		}
 	}
-	*end_offset = vte_char_attr_list_get_size(&priv->snapshot_attributes) - 1;
-	for (i = offset + 1; i < vte_char_attr_list_get_size(&priv->snapshot_attributes); i++) {
-                cur_attr = *vte_char_attr_list_get(&priv->snapshot_attributes, i);
+	*end_offset = priv->snapshot_attributes->len - 1;
+	for (i = offset + 1; i < priv->snapshot_attributes->len; i++) {
+		cur_attr = g_array_index (priv->snapshot_attributes,
+				      struct _VteCharAttributes,
+				      i);
 		if (!_pango_color_equal (&cur_attr.fore, &attr.fore) ||
 		    !_pango_color_equal (&cur_attr.back, &attr.back) ||
 		    cur_attr.underline != attr.underline ||
@@ -1454,7 +1349,7 @@ vte_terminal_accessible_get_character_count(AtkText *text)
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
 
-	return vte_char_attr_list_get_size(&priv->snapshot_attributes);
+	return priv->snapshot_attributes->len;
 }
 
 static gint
@@ -1687,8 +1582,8 @@ vte_terminal_accessible_set_size(AtkComponent *component,
         /* If the size is an exact multiple of the cell size, use that,
          * otherwise round down. */
         try {
-                width -= impl->m_border.left + impl->m_border.right;
-                height -= impl->m_border.top + impl->m_border.bottom;
+                width -= impl->m_padding.left + impl->m_padding.right;
+                height -= impl->m_padding.top + impl->m_padding.bottom;
 
                 auto columns = width / impl->m_cell_width;
                 auto rows = height / impl->m_cell_height;
